"use strict";
var pretur_redux_1 = require('pretur.redux');
var Querier_1 = require('./query/Querier');
var actions_1 = require('./data/actions');
var INIT_KEY = '@@CLAY_REACTIVE_INIT';
function isInit(action) {
    return action[INIT_KEY];
}
function informInit(action) {
    action[INIT_KEY] = true;
    return action;
}
var Reactive = (function () {
    function Reactive(set, query, buildFetcher) {
        if (buildFetcher === void 0) { buildFetcher = null; }
        if (set === null) {
            return;
        }
        this._set = set;
        this._querier = new Querier_1.default(query);
        this._initialized = false;
        this._buildFetcher = buildFetcher;
    }
    Object.defineProperty(Reactive.prototype, "set", {
        get: function () {
            return this._set;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "items", {
        get: function () {
            return this._set.items;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "querier", {
        get: function () {
            return this._querier;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "initialized", {
        get: function () {
            return this._initialized;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Reactive.prototype, "valid", {
        get: function () {
            return this._set.valid;
        },
        enumerable: true,
        configurable: true
    });
    Reactive.prototype.reduce = function (action) {
        var _this = this;
        var newSet = this._set.reduce(action);
        var newQuerier = this._querier.reduce(action);
        if (newSet !== this._set || newQuerier !== this._querier) {
            var clone_1 = this.clone();
            clone_1._set = newSet;
            clone_1._querier = newQuerier;
            if (isInit(action)) {
                clone_1._initialized = true;
            }
            else if (this._buildFetcher && newQuerier !== this._querier) {
                pretur_redux_1.emit(action, function (dispatch) {
                    var fetcher = _this._buildFetcher();
                    clone_1.forceRefresh(dispatch, fetcher);
                    fetcher.fetch();
                });
            }
            return clone_1;
        }
        return this;
    };
    Reactive.prototype.init = function (dispatch, fetcher) {
        return this._set.refresh(pretur_redux_1.hook(dispatch, informInit), this._querier.plain, fetcher);
    };
    Reactive.prototype.initIfNeeded = function (dispatch, fetcher) {
        if (!this._initialized) {
            this.init(dispatch, fetcher);
        }
    };
    Reactive.prototype.forceRefresh = function (dispatch, fetcher) {
        return this._set.refresh(dispatch, this._querier.plain, fetcher);
    };
    Reactive.prototype.ensureValidity = function (dispatch) {
        if (this.valid) {
            return true;
        }
        dispatch(actions_1.CLAY_DATA_DECAY.create.broadcast());
        return false;
    };
    Reactive.prototype.sync = function (synchronizer) {
        this._set.appendSynchronizationModels(synchronizer);
    };
    Reactive.prototype.ensureValidityAndSync = function (dispatch, synchronizer, autoDecay) {
        if (autoDecay === void 0) { autoDecay = true; }
        if (this.valid) {
            this.sync(synchronizer);
            return true;
        }
        if (autoDecay) {
            dispatch(actions_1.CLAY_DATA_DECAY.create.broadcast());
        }
        return false;
    };
    Reactive.prototype.clone = function () {
        var clone = new Reactive(null, null);
        clone._set = this._set;
        clone._querier = this._querier;
        clone._initialized = this._initialized;
        clone._buildFetcher = this._buildFetcher;
        return clone;
    };
    return Reactive;
}());
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = Reactive;
